/**
** This software is distributed under the terms of the General Public License.  
** 
** Program : BuildIT!
** Version : 0.2
** Authors : Artur Prusakiewicz <prusak@mat.uni.torun.pl>
**	     Michal Switala <scisly@mat.uni.torun.pl>
** E-MAIL  : build@mat.uni.torun.pl
** WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
**           http://www.mat.uni.torun.pl/~prusak/buildit/
**/

#include "Dialog.h"

Dialog::Dialog(QWidget *parent, const char *name):QListView(parent,name)
{
  int i;

  canTouch=TRUE;
  quan=0;

  for(i=0;i<NUMPROJECTS;i++)
    {
      navi[i]=NULL;
      items[i]=NULL;
    }
  connect(this,SIGNAL(currentChanged(QListViewItem *)),SLOT(touch(QListViewItem *)));
  connect(this,SIGNAL(doubleClicked(QListViewItem *)),parent,SLOT(prefs()));

  resize(WIDTH,200);
  addColumn("Project Name");
  addColumn("Project Path");
  addColumn("Last Modified");
}

void Dialog::addProject(MyClass *widget)
{
  QString tmp;
  QListViewItem *item; 
  int i=0;
  canTouch=FALSE;

  QString parse(widget->name());
  tmp=parse.right(parse.length()-parse.find(" "));
  item=new QListViewItem(this);
  item->setText(0,tmp.data());
  item->setText(1,widget->getCpt().data());

  QFileInfo *fInfo=new QFileInfo(widget->getFileName().data());
  item->setText(2,fInfo->lastModified().toString());

  setSelected(item,TRUE);
  navi[quan]=widget;
  items[quan]=item;
  quan++;

  canTouch=TRUE;
}

bool Dialog::isOpen(QString val)
{
  int i;

  for(i=0;i<quan;i++)
      if(navi[i]->getFileName()==val)
	return FALSE;

  return TRUE;
}

void Dialog::changeProject(MyClass *widget)
{
  QString tmp;
  QListViewItem *item;
  int i,j;

  canTouch=FALSE;

  QString parse(widget->name());
  tmp=parse.right(parse.length()-parse.find(" "));

  for(i=0;i<quan;i++)
    if(widget==navi[i])
      {
	item=items[i];
	item->setText(0,tmp.data());
	item->setText(1,widget->getCpt().data());
	
	QFileInfo *fInfo=new QFileInfo(widget->getFileName().data());
	item->setText(2,fInfo->lastModified().toString());
	setSelected(item,TRUE);
      }
  canTouch=TRUE;  
}

bool Dialog::rmProject(MyClass *widget)
{
  QString tmp,item;
  int i,j,num=-1;

  if(quan==0)
    return FALSE;

  canTouch=FALSE;
  for(i=0;i<quan;i++)
    if(navi[i]==widget)
      {
	delete(items[i]);

	for(j=i;j<quan;j++)
	  {
	    navi[j]=navi[j+1];
	    items[j]=items[j+1];
	  }
	quan--;
      }
  canTouch=TRUE;  
  if(quan>0)
    {
      setSelected(items[0],TRUE); 	  
      navi[0]->raiseIT();
      navi[0]->setWidgetTouched(navi[0]); 
    }
  return TRUE;
}

void Dialog::setHighlighted(MyClass *widget)
{
  int i;
  for(i=0;i<quan;i++)
    if(navi[i]==widget)
      setSelected(items[i],TRUE); 
}

void Dialog::touch(QListViewItem *widget)
{
  int i;

  if(canTouch)
    {
      for(i=0;i<quan;i++)
	if(items[i]==widget)
	  {
	    navi[i]->raiseIT();
	    navi[i]->setWidgetTouched(navi[i]);
	  }
    }
}

void Dialog::end()
{
  int i;

  for(i=0;i<quan;i++)
    delete(navi[i]);
}










