/**
** This software is distributed under the terms of the General Public License.  
** 
** Program : BuildIT!
** Version : 0.2
** Authors : Artur Prusakiewicz <prusak@mat.uni.torun.pl>
**	     Michal Switala <scisly@mat.uni.torun.pl>
** E-MAIL  : build@mat.uni.torun.pl
** WWW     : http://www.mat.uni.torun.pl/~scisly/buildit/
**           http://www.mat.uni.torun.pl/~prusak/buildit/
**/

#include "Tree.h"
#include "Generator.h"
#include <stdio.h>


bool MyTree::CutFileData(QString buf, QString *t1[], int t2[])
{
  int pos1, pos2, i;
  QString tmpbuf(200);
  QString str(30);

  pos1 = 0;
  pos2 = 1;
  for (i = 0; i < NUMFEATURE1; i++)
    {
      tmpbuf = buf.copy();
      pos1 = tmpbuf.find("~", pos1, TRUE);
      pos2 = tmpbuf.find("~", pos2, TRUE);
      tmpbuf.truncate(pos2);
      tmpbuf.remove(0, pos1+1);
      *t1[i] = tmpbuf.copy();
      pos1 = pos2;
      pos2++;
    }
  for (i = 0; i < NUMFEATURE2; i++)
    {
      tmpbuf = buf.copy();
      pos1 = tmpbuf.find("~", pos1, TRUE);
      pos2 = tmpbuf.find("~", pos2, TRUE);
      tmpbuf.truncate(pos2);
      tmpbuf.remove(0, pos1+1);
      t2[i] = tmpbuf.toInt();
      pos1 = pos2;
      pos2++;
    }
  return TRUE;
}

int MyTree::CutIdString(QString* base, QString* result[])
{
  int pos, count, i, finish;

  count = 0;
  QString *t = new QString(30);
  QString *parts[30];
  for (i = 0; i < 30; i++) 
    parts[i] = new QString(5);
  finish = 1;
  count = 1;
  pos = -1;
  *t = base->copy();
  *parts[0] = base->copy();
  while (finish)
    {
      pos = t->findRev('|', pos);
      t->truncate(pos);
      if (pos != -1) 
	 {
	   *parts[count] = t->copy();
	   count++;
	 }
      else 
	finish = 0;
    }
  count--;
  if (t) 
    delete t;
  for (i = count; i >= 0; i--) 
    *result[count-i] = *parts[i];
  for (i = 0; i < 30; i++) 
    if (parts[i]) 
      delete parts[i];
  return count;
}

MyTree::MyTree()
{
  int i;

  for (i=0; i < NUMFEATURE1; i++) 
    feature1[i] = new QString(30);
  for (i=0; i < MAXCHILDREN; i++) 
    children[i] = NULL;
  numchildren = 0;
  parent = NULL;
  terror = 0;
}


MyTree::~MyTree()
{

}

bool MyTree::IncChildren()
{
  if (GetNumChildren() < MAXCHILDREN)
    numchildren++;
  else 
    {
      terror = 1;
      return FALSE;
    }
  return TRUE;
}

bool MyTree::DecChildren()
{
  if (GetNumChildren() > 0) 
    numchildren--;
  else 
    return FALSE;
  return TRUE;
}

MyTree* MyTree::Find(QString* id)
{
  MyTree *start = this;
  if ((id->contains('|')) == 0) 
    {
     if (*id == *start->GetFeature1(0)) 
	 return start;
      else 
	 return NULL;
    }
   else
     {
       QString *parts[30];
       MyTree* temp;
       int howmany, finish, which;

       for (int i = 0; i < 30; i++) 
	 parts[i] = new QString(5);
       howmany = CutIdString(id, parts);
       finish = 1;
       temp = start->ptr;
       which = 1;
       while (finish)
	 {
	   int i;
	   for (i = 0; i < temp->GetNumChildren(); i++) 
	       {
		 MyTree *t2;
		 t2 = temp->children[i];
		 if (*((temp->children[i])->GetFeature1(0)) == *parts[which]) 
		   if (which == howmany) 
		     {
		       for (int ii = 0; ii < 30; ii++) 
			 if (parts[ii]) 
			   delete parts[ii];
		       return (temp->children[i])->ptr;
		     }
		    else 
		      {
			temp = temp->children[i];
			i = 0;
		      }
	       }
	   if (which == howmany) 
	     {
	       for (int ii = 0; ii < 30; ii++) 
		 if (parts[ii]) 
		   delete parts[ii];
	       return NULL;
	     }
	   which++;
	 }
     }
  return NULL;
}

MyTree* MyTree::FindParent(QString* id)
{
  QString *tmpid = new QString(30);
  int where;

  *tmpid = id->copy();
  where = tmpid->findRev("|");

  tmpid->truncate(where);
  MyTree *tmp;

  tmp = this->Find(tmpid);
  if (tmpid) 
    delete tmpid;

  return tmp;
}

bool MyTree::Update(QString* tab1[], int tab2[])
{
  int i;
  MyTree *temp;

  temp = this->Find(tab1[0]);
  if (temp == NULL)
    {
      printf("update error!\n");
      terror = 2;
      return FALSE;
    }
   else
     {
       for (i = 0; i < NUMFEATURE1; i++) 
	 temp->SetFeature1(i, tab1[i]);
       for (i = 0; i < NUMFEATURE2; i++) 
	 temp->SetFeature2(i, tab2[i]);
       return TRUE;
     }
}

bool MyTree::Add(QString* tab1[], int tab2[])
{
  int i;
  MyTree *tmp = new MyTree();

  tmp->ptr = tmp;
  if ((tab1[0]->contains("|")) == 0) 
    {
      tmp = this;     
      tmp->ptr = this;
      tmp->parent = NULL;
      tmp->root = this;  

      for (i = 0; i < NUMFEATURE1; i++) 
	*(tmp->feature1[i]) = tab1[i]->copy();
      for (i = 0; i < NUMFEATURE2; i++) 
	tmp->feature2[i] = tab2[i];
      return TRUE;
    }
  else 
    {
      QString *tmpstr = new QString(30);
      *tmpstr = tab1[0]->copy();
      tmp->parent = FindParent(tmpstr);
      delete tmpstr;
      tmp->parent->IncChildren();
      tmp->parent->children[(tmp->parent->numchildren)-1] = tmp;
      tmp->root = NULL;
    }
  for (i = 0; i < NUMFEATURE1; i++) 
    *(tmp->feature1[i]) = tab1[i]->copy();
  for (i = 0; i < NUMFEATURE2; i++) 
    tmp->feature2[i] = tab2[i];
  return TRUE;
}     

bool MyTree::RemoveLeaf(MyTree *p)
{
  int i;
  int ch = p->GetNumChildren();

  if (ch == 0)
     {
       for (i = 0; i < NUMFEATURE1; i++) 
         if (p->feature1[i] != NULL) 
	   delete p->feature1[i];
       p->numchildren = 0;
       return TRUE;
     }
    else
      {
	for (i = 0; i < ch; i++)
	  p->RemoveLeaf(p->children[i]);
	if (p->root == NULL)
	  {
	    p->numchildren = 0;
	    for (i = 0; i < NUMFEATURE1; i++) 
	      if (p->feature1[i] != NULL) 
		delete p->feature1[i];
	    for (i = 0; i < NUMFEATURE2; i++) 
	      p->SetFeature2(i, 0);
	    return TRUE;
	  }
	else
	  {
	    p->numchildren = 0;
	    for (i = 0; i < NUMFEATURE1; i++) 
	      p->SetFeature1(i, "0");
	    for (i = 0; i < NUMFEATURE2; i++) 
	      p->SetFeature2(i, 0);
	    p->root = NULL;
	    return FALSE;
	  }
      }
  return TRUE;
}


bool MyTree::Remove(QString* id)  
{
  MyTree *temp;
  QString tempid;
  tempid = id->copy();

  temp = Find(id);

  if (temp == NULL) 
    return FALSE;
  if (temp->GetNumChildren() != 0)
     if (temp->RemoveLeaf(temp) == FALSE) 
       return TRUE;
  temp->numchildren = 0;
  MyTree *parent_ptr;

  parent_ptr = FindParent(&tempid);
 
  if (parent_ptr != NULL) 
    {    
      if (parent_ptr->GetNumChildren() == 0) 
	return TRUE;
      int i1, i2;
      int pch = parent_ptr->GetNumChildren();
      for (i1 = 0; i1 < pch; i1++)
        {
	  if (parent_ptr->children[i1] == temp)
	    {
	      for (i2 = i1; i2 < pch - 1; i2++)
		parent_ptr->children[i2] = parent_ptr->children[i2 + 1]; 
	      parent_ptr->DecChildren();
	      
	      return TRUE;
	    }
	}
    }
   else 
     return TRUE;
  terror = 3;
 
  return FALSE;
}

QString* MyTree::GetFeature1(int which)
{
  if ((which >= 0) && (which < NUMFEATURE1)) 
    return feature1[which];
  else 
    {
      terror = 4;
      return NULL;
    }
}

int MyTree::GetFeature2(int which)
{
  if ((which >= 0) && (which <= this->feature2[0])) 
    return this->feature2[which];
  else
    {
      terror = 5;
      return -1;
    }
}

bool MyTree::SetFeature1(int which, QString* what)
{
  if ((which >= 0) && (which < NUMFEATURE1)) 
    *feature1[which] = what->copy();
  else 
    {
      terror = 6;
      return FALSE;
    }
  return TRUE;
}

bool MyTree::SetFeature1(int which, QString what)
{
  if ((which >= 0) && (which < NUMFEATURE1)) 
    *feature1[which] = what.copy();
  else 
    {
      terror = 6;
      return FALSE;
    }
  return TRUE;
}

bool MyTree::SetFeature2(int which, int what)
{
  if ((which >= 0) && (which < NUMFEATURE2)) 
     {
       feature2[which] = what;
       return TRUE;
     }
  terror = 7;
  return FALSE;
}

bool MyTree::WriteLeaf(MyTree *p, QFile *f)
{
  int i;
  QString breakstring = "~";
  f->writeBlock(breakstring, 1);
  for (i = 0; i < NUMFEATURE1; i++)
      {
	int len = p->GetFeature1(i)->length();

	if (f->writeBlock((const char*) *p->GetFeature1(i), len) != len)
	   {
	     printf("Tree::WriteLeaf error => writeBlock to file \n");
	     terror = 8;
	     return FALSE;
	   }
	f->writeBlock(breakstring, 1);
      }
  for (i = 0; i < NUMFEATURE2; i++)
    {
      QString str(10);
      str = str.setNum(p->GetFeature2(i));
      int len = str.length();
      
      if (f->writeBlock((const char*) str, len) != len)
	{
	  printf("Tree::WriteLeaf error => writeBlock to file \n");
	  terror = 8;
	  return FALSE;
	}
      f->writeBlock(breakstring, 1);
    }
  f->writeBlock("\n", 1);
  int numchild;
  numchild = p->GetNumChildren();
  
  if (numchild == 0) 
    return TRUE;
  for (i = 0; i < numchild; i++) 
    if (WriteLeaf(p->children[i], f) == FALSE) 
      return FALSE;
  return TRUE;
}

bool MyTree::Tree2File(QString filename, QString home)
{
  QFile f;

  f.setName(filename);
  if (f.open(IO_WriteOnly) == FALSE) 
    {
      printf("Tree::Tree2File error => cannot open file %s to write\n",filename.data());
      return FALSE;
    }
  WriteLeaf(this, &f);
  f.flush();
  int i;
  QString *res[10];
  for (i=0; i<10; i++) res[i] = new QString(20);
  int n = 0;
  FindFeature(this, 2, QString("MyTextArea"), 1, res, n);
  if (n)
    {
      QString buf;
      buf = "<TEXTAREA SECTION> " + QString().setNum(n) + "\n";
      f.writeBlock((const char*) buf, buf.length());
      for (i = 0; i < n; i++) 
	{
	  QString temp1=res[i]->copy();
	  int pos=temp1.findRev(" ");
	  int hm=temp1.length()-pos;
	  temp1 = temp1.right(hm-1);
	  QString name = home + ".build/tmp/" + temp1.copy() + ".tmp";
	  buf = temp1.copy() + "\n";
	  f.writeBlock((const char*)buf, buf.length());
	  QFile f2(name);
	  if (f2.open(IO_ReadOnly) == FALSE)
	    {
	      buf = "<<<end>>>\n";
	      f.writeBlock((const char*) buf, buf.length());
	    }
	  else 
	    {
	      QTextStream t( &f2 );
	      QString s;          
	      while ( !t.eof() ) 
		{
		  s = t.readLine(); 
		  if (s.length() > 0) s.append("\n");
		  f.writeBlock(s, s.length());
		}  
	      f2.close();
	      buf = "<<<end>>>\n";
	      f.writeBlock((const char*)buf, buf.length());
	    }
	}
    }
  f.close();
  return TRUE;
}

bool MyTree::File2Tree(QString filename, QString home)
{
  int i;
  QFile f;

  f.setName(filename);
  if (f.open(IO_ReadOnly) == FALSE) 
    {
      printf("Tree::File2Tree error => cannot open file %s to read\n",filename.data());
      return FALSE;
    }
  QString buf(1000);
  char *b = new char[1000];
  while (!f.atEnd())
    {
      if (f.readLine(b, 1000) != 0)
	{     
	  if (QString(b).contains("<TEXTAREA SECTION>") == 0)
	    {
	      buf.insert(0, b);
	      QString *tab1[NUMFEATURE1];
	      int tab2[NUMFEATURE2];
	      for (i = 0; i < NUMFEATURE1; i++) 
		tab1[i] = new QString(30);
	      buf=CutFileData(buf, tab1, tab2);
	      Add(tab1, tab2);
	      for (i = 0; i < NUMFEATURE1; i++) 
		if (tab1[i]) 
		  delete tab1[i];
	    }
	  else
	    {
	      QString line(b);
	      line.remove(0, 18);
	      int sections = line.toInt();
	      int ii;
	      for (ii=0; ii<sections; ii++)
		{
		  QFile fout;
		  QString name(home+".build/tmp/");
		  if (f.readLine(b, 200) != 0) 
		    {
		      name.append(b);
		      name = name.stripWhiteSpace(); 
		      name.append(".tmp");
		      name = name.simplifyWhiteSpace();
		      fout.setName(name);
		      if (fout.open(IO_WriteOnly) == FALSE) 
			{
			  printf("Tree::Tree2File error => cannot open file %s to write\n",name.data());
			  return FALSE;
			}
		      QTextStream t( &f );
		      QString s;          
		      bool endit = FALSE;
		      while (!endit)  
			{
			  s = t.readLine(); 
			  if (s.contains("<<<end>>>") == 0)
			    {
			      s.append("\n");
			      fout.writeBlock(s, s.length());
			    }
			  else endit = TRUE;
			}
		      fout.close();
		    }
		}
	    }  
	}
    }
  if (b) delete b;
  return TRUE;
}

bool MyTree::Show(MyTree *p)
{
  int i, k,j;

  printf("---------F1--------\n");
  for (k=0; k<NUMFEATURE1; k++)
    printf("F1[%d]=%s\n", k, (const char*) *p->GetFeature1(k));
  printf("-------- F2----------\n");
  for (k=0; k<NUMFEATURE2; k++)
    printf("F2[%d]=%d\n", k, p->GetFeature2(k));
  int numchild;
  numchild = p->GetNumChildren();
  if (numchild == 0)
    return TRUE;
  for (i = 0; i < numchild; i++) 
    if (Show(p->children[i]) == FALSE) 
      return FALSE;
  return TRUE;
}

int MyTree::GetNumAllChildren()
{
  int i, numch, result;

  numch = GetNumChildren();
  if (numch == 0) 
    return 0;
  result = numch;
  for (i = 0; i < numch; i++)
    result += children[i]->GetNumAllChildren();
  return result;
}

MyTree* MyTree::findFeature1(MyTree *t, int which, QString *what)
{
  if (t->GetFeature1(which)->contains((const char*) *what) > 0)
    return t;
   else
     {
       int i;
       int numch = t->GetNumChildren();
       if (numch == 0) 
	 return NULL;
       MyTree *result;
       for (i = 0; i < numch; i++)
	 {
	   result = findFeature1(t->GetChildren(i), which, what);
	   if (result != NULL) 
	     return result;
	 }
       return NULL;
     }
}

bool MyTree::FindFeature(MyTree *t,int where, QString name,int what, QString *result[], int &n)
{
  int i, j, numch;

  if (*t->GetFeature1(where) == name)
    {
      bool found;
      found = FALSE;
      for (j = 0; j < n; j++) 
	if (*result[j] == *t->GetFeature1(what)) 
	  found = TRUE;
      if (!found)
	*result[n++] = t->GetFeature1(what)->copy();
    }
  numch = t->GetNumChildren();
  if (numch == 0) 
    return TRUE;
  for (i = 0; i < numch; i++)
    FindFeature(t->GetChildren(i), where, name, what, result, n);
  return TRUE;
}

bool MyTree::countSub(MyTree *t,int &n)
{
  int i,numch;
  bool found=FALSE;

  n++;
  numch = t->GetNumChildren();
  if (numch == 0) 
    return TRUE;
  for (i = 0; i < numch; i++)
    countSub(t->GetChildren(i),n);
  
  return TRUE;
}

bool MyTree::getSub(MyTree *t,QString *result,int &n)
{
  int i, j, numch;
  bool found=FALSE;

  for (j = 0; j < n; j++) 
    if (result[j] == *t->GetFeature1(0)) 
      found = TRUE;
  if (!found)
    result[n++] = t->GetFeature1(0)->copy();

  numch = t->GetNumChildren();
  if (numch == 0) 
    return TRUE;
  for (i = 0; i < numch; i++)
    getSub(t->GetChildren(i),result, n);
  
  return TRUE;
}

bool MyTree::getItems(QString *result)
{
  int i,j=0;
  
  result[j++]=GetFeature1(0)->copy();
  for (i=0;i<GetNumChildren();i++)
    {
      result[j++]=GetChildren(i)->GetFeature1(0)->copy();
    }

  return TRUE;
}

void MyTree::getXY(QString ID,int &x,int &y)
{
  MyTree *tempTree;
  int i,j=2;
  QString currID;
  
  for(i=1;i<ID.contains("|");i++)
    {
      currID=ID.left(ID.find("|",j));
      j=currID.length()+1;
      tempTree=this->Find(&currID);
      x+=tempTree->GetFeature2(1);
      y+=tempTree->GetFeature2(2);
    }
}

bool MyTree::countFeature(MyTree *t,int which, int &sum)
{
  if (t->GetFeature1(2)->copy() != QString("MyClass"))
    sum += t->GetFeature2(which);
  if (t->GetParent() != NULL)
    countFeature(t->GetParent(), which, sum);
  return TRUE;
}
